
*-----------------------------------------------------------
* Program    : Bitmap Drawer
* Written by : Alex Lemke
* Date       : Oct 6
* Description: Turret Game, Hit Q to Quit.
*
* Space = Fire
* A, D = Left, Right
* W, S = Adjust Turret Angle
* 
* Program can handle mutliple enemies.  
*	Currently set to one enemy for best framerate
*-----------------------------------------------------------

ALL_REG                 REG     D0-D7/A0-A6

SCREEN_WIDTH		equ	80
SCREEN_HEIGHT		equ	70
SET_OUTPUT_WINDOW_SIZE	equ	33

SET_DRAWING_MODE	equ	92
DOUBLE_BUFFERED_MODE	equ	17
SWAP_BUFFERS		equ	94

GET_KEY_INPUT_COMMAND   equ     19
GET_CYCLE_COUNT		equ	31
GET_TIME		equ	8

*******************************************
*Draw EQUs
*******************************************
CLEAR_SCREEN	equ	11

SET_PEN_COLOR 	equ 	80
SET_FILL_COLOR	equ	81

DRAW_PIXEL	equ	82
DRAW_LINE	equ	84
DRAW_RECTANGLE	equ	87
DRAW_ELLIPSE	equ	88

******************************************
*Colors
******************************************
RED	equ	$000000FF
GREEN	equ	$0000FF00
BLUE	equ	$00FF0000
YELLOW	equ	RED+GREEN
ORANGE	equ	$003090FF
BLACK	equ	$00000000

*******************************************
*Tank 
*******************************************

TANK_Y			equ	798000

TANK_WIDTH		equ	16
TANK_HEIGHT		equ	30
TANK_MOVEMENT_SPEED	equ	6144 *1.5




************************************************
*Turret
************************************************
TURRET_COLOR		equ	RED
TURRET_LENGTH		equ	25



************************************************
*Power Meter
************************************************

POWER_METER_SHELL_COLOR		equ	GREEN
POWER_SHELL_FCOLOR		equ	BLUE
POWER_METER_FILL_COLOR		equ	ORANGE




POWER_METER_X		equ	360
POWER_METER_Y		equ	20
POWER_METER_WIDTH	equ	250
POWER_METER_HEIGHT	equ	13


POWER_METER_SPEED	equ	10

***********************************************
*Gravity
***********************************************
GRAVITY			equ	1000


***********************************************
*Seven Segment LED
**********************************************
*
*	 AAAAAAA
*	F	B
*	F	B
*	F	B
*	F	B
*	 GGGGGGG
*	E	C
*	E	C
*	E	C
*	E	C
*	 DDDDDDD
*
*Bit Number  76543210 
*Segment     xABCDEFG

TEXT_COLOR	equ	ORANGE
SCORE_LOCATION_X	equ	65
SCORE_LOCATION_Y	equ	5

****************************************************
* Enemies
****************************************************
NUMBER_OF_ENEMIES	equ	1
*Use One Enemy For Best Frame Rate

**************************************************
* Sounds
**************************************************

LOAD_WAV		equ	71
PLAY_LOADED_WAV		equ	72


START	ORG	$1000

*Clearing the Screen
	move.w	#$FF00, d1	
	trap	#15	

*Resizing the  Screen			
	move.b	#SET_OUTPUT_WINDOW_SIZE, d0	*Setting the output window size
	move.l  #SCREEN_WIDTH*$10000+SCREEN_HEIGHT, d1       
	trap	#15

*Setting Drawing Mode
	move.b	#SET_DRAWING_MODE, d0
	move.b 	#17, d1
	trap	#15


	move.l  #409600, TANK_X_CURRENT
	move.l  #409600, TANK_X_PREVIOUS



	move.l	#0, TANK_VELOCITY_X
	move.l	#0, POWER_METER_VALUE
	move.w	#90, TURRET_ANGLE_CURRENT

	move.l	#TANK_Y, TANK_Y_CURRENT




*Score init
	move.l	#00, BULLET_COUNTER	
	move.l	#00, HIT_COUNTER	
*Bullet init
	move.l	#$1EA000, d0
	move.l	d0, BULLET_Y
	move.l	#0, BULLET_FIRED
	move.l	#0, BULLET_FRAME


	move.b	#GET_TIME, d0
	trap	#15
	move.l	d1, CURRENT_TIME
	


	
	lea	Music_1,a1 *load music 1
	move	#0,  d1	*Move music 1 to sound 0
	move	#LOAD_WAV, d0
	trap	#15	*play sound 0

	




*********************************************************
*Inital Background Draw
*********************************************************
	pea	backgroundData			*Push the address of the bitmap onto the stack
	
	move.w #0, -(SP)			*Push parameter positionX on to the stack
    	move.w #0, -(SP)			*Push parameter positionY on to the stack

	move.w #320, -(SP)			*Push parameter the width on to the stack
    	move.w #240, -(SP)			*Push parameter the height on to the stack

	move.w #0, -(SP)			*Push parameter draw screen position x
    	move.w #0, -(SP)			*Push parameter draw screen position y

	jsr	drawBMP				*Jump to sub routine drawBMP

	add.l #16, SP           		*Remove the passed parameters from the stack, all at once. (4 + (2 * 4))



	move.w	#0 ,d1	*play sound 1
	move.b	#PLAY_LOADED_WAV, d0
	trap	#15


*************************************************************
*Game Loop
*************************************************************
gameLoopStart:


*Previous Time  = Current Time
	move.l	CURRENT_TIME, PREVIOUS_TIME

*Getting the Current Time	
	move.b	#GET_TIME, d0
	TRAP 	#15
	move.l	d1, CURRENT_TIME
	
*Getting Elpased Time	
	move.l	 PREVIOUS_TIME, d2
	sub.l	d2, d1
	move.l	d1, ELASPED_TIME



*Update
	jsr	update

*Input
	jsr	handleInput

*Render
	jsr	render

	
	jmp	gameLoopStart




**********************************************************
*Input
**********************************************************
handleInput:
	movem.l ALL_REG, -(sp)
	
*ASCII CODE 31 = 0
        move.l          #$20,d2
nextKey:
	move.b          #GET_KEY_INPUT_COMMAND, d0
*put current ascii value we're looking for into d1 for trap
        move.l          d2, d1
        TRAP            #15
*if key is pressed call function if not just goto next ascii value
        cmpi.b          #0,d1
        beq             noCall
        jsr             callFunction
noCall
*bump to next ascii value
        addi.l           #1,d2
*if we're at Z, then reinit the data to 0 and begin loop again
        cmpi.b          #$5A,d2
        bne             nextKey


	jmp	escape_Input

	
	


escape_Input
	movem.l (sp)+,ALL_REG
	rts



********************************************************************
*Call Function
********************************************************************
callFunction:
*save off registers
        movem.l ALL_REG,-(sp)
*load up KeyInputFunctionTable[d2-'0']  
        lea     KeyInputFunctionTable,a0
        sub.l   #$20,d2
        lsl.l   #2,d2
        move.l  (a0,d2),d1
*if it's a null function ptr, nothting to call so leave
        cmpi.l  #0,d1
        beq     noFuncPtr
*move value into A1 and call it
        move.l  d1,a1
        jsr     (a1)    
noFuncPtr
        movem.l (sp)+,ALL_REG
        rts


*************************************************************
*input routines
*************************************************************

routineSpace
	
	move.l	BULLET_FIRED, d0
	bgt	cantFire


*X1
	move.l	TANK_X_CURRENT, d1	
	lsl.l	#4, d1
	swap	d1

	*move.l	TANK_X_CURRENT, d1	
	*lsr.l	#8, d1
	*lsr.l	#4, d1

*Saves 20 Cycles Per Shift by doing  lsrl by 1 and then a swap instead of lsr.l	#8 and lsr.l #4
	move.w	#TANK_WIDTH, d2
	lsr.l	#1, d2
	add.l	d2, d1
	
*Y1	
	move.l	TANK_Y_CURRENT, d2
	lsl.l	#4, d2
	swap	d2
	*lsr.l	#8, d2
	*lsr.l	#4, d2
	
	lea	SIN_TABLE, a3	
	lea	COS_TABLE, a4	

	move.w	TURRET_ANGLE_CURRENT, d7
	asl.l	#1, d7
*X2
	move.l	d1, d3
	move.l	#0, d5
	move.w	(a3, d7), d5

	mulu	#TURRET_LENGTH, d5


	*lsr.l	#8, d5
	*lsr.l	#4, d5
	
	
	lsl.l	#4, d5
	swap	d5

	add.l	d5, d3
*Y2
	move.l	d2, d4
	move.l	#0, d6

	move.w	(a4, d7), d6
	
	mulu	#TURRET_LENGTH, d6
	*mulu	d0, d6

	lsr.l	#8, d6
	lsr.l	#4, d6
	*lsl.l	#4, d6
	*swap	d6

	sub.w	d6, d4




	lsl.l	#8, d3
	lsl.l	#4, d3
	*lsl.l	#4, d3
	*swap	d3
	
	lsl.l	#8, d4
	lsl.l	#4, d4
	*lsl.l	#4, d4
	*swap	d4

	move.l	d3,  BULLET_X
	move.l	d4,  BULLET_Y


	clr.l	BULLET_VELOCITY_X
	clr.l	BULLET_VELOCITY_Y
	

	
	lsl.l	#8, d1
	lsl.l	#4, d1
	*lsl.l	#4, d1
	*swap	d1
	
	lsl.l	#8, d2
	lsl.l	#4, d2
	*lsl.l	#4, d2
	*swap	d2
	
	sub.l	d1, d3
	sub.l	d2, d4





	move.w	TURRET_ANGLE_CURRENT, d7
	asl.l	#1, d7
*X2



*POWER METER TIME!


	move.l	#0, d5
	
*Getting the sin and cos values from the sin table


	move.w	(a3, d7), d5
	move.w	(a4, d7), d6


*Getting the value of the power meter
	move.l	POWER_METER_VALUE, d0


*Pre shifting to avoid over flow
	lsr.l	#8, d0
	lsr.l	#8, d5
	lsr.l	#8, d6

*Scaling the effect of the power meter
	*move.l	ELASPED_TIME, d7	
	mulu	#10, d0
	*mulu	 d7, d0
	
*Multiplying the x and y values by the amoutn of power

	muls	d0, d5
	muls	d0, d6

*result of fixed * fixed is fixed squared so divide out one fixed conversion
*by dividing by 16

	lsr.l	#4, d5
	lsr.l	#4, d6
	

*The y value needs to be negative so mulu it by -1
	muls	#-1, d6

*Shifting back due to pre shifting
	lsl.l	#8, d5
	lsl.l	#8, d6

	move.l	d5, BULLET_VELOCITY_X
	move.l	d6, BULLET_VELOCITY_Y

	move.l	#0, POWER_METER_VALUE

	move.l	#1, BULLET_FIRED 
	addi.l	#1, BULLET_COUNTER

cantFire:
        rts

routineA
*Moving player to the left
	sub.l	#TANK_MOVEMENT_SPEED,  TANK_VELOCITY_X
        rts

routineD
*Moving player to the right
	addi.l	#TANK_MOVEMENT_SPEED,  TANK_VELOCITY_X 
        rts



routineQ
       * movem.l ALL_REG,-(sp) 

	jmp ProgramDone
  
       * movem.l (sp)+,ALL_REG
        rts



routineS
*Lowering the turret
*0 = straight up
*90 = straight across
	move.w	TURRET_ANGLE_CURRENT, d1
	move.w	#1, d2
	move.l	ELASPED_TIME, d3
	mulu	d3, d2


	add.w	d2, TURRET_ANGLE_CURRENT
	move.w	TURRET_ANGLE_CURRENT, d1
	
	move.w	#90, d2
	
	cmp.w	d1, d2    	
	bgt	validAngleUp
	
	move.w	#90, TURRET_ANGLE_CURRENT


validAngleUp:
        rts

routineW
*Rasing the turret
*0 = straight up
*90 = straight across

	move.w	TURRET_ANGLE_CURRENT, d1
	move.w	#1, d2
	move.l	ELASPED_TIME, d3
	mulu	d3, d2


	sub.w	d2, TURRET_ANGLE_CURRENT
	move.w	TURRET_ANGLE_CURRENT, d1
   

	move.w	#0, d0
	
	cmp.w	d0, d1    	
	bgt	validAngleDown
	
	move.w	#00, TURRET_ANGLE_CURRENT


validAngleDown:
        rts













****************************************************
*Updates
****************************************************
update:
	movem.l ALL_REG, -(sp)
	
	
	
	
	*We do jumps instead of pushing the return value onto the stack.
	*This is done to save some time but its bad programming
	*(pro: minor clock savings)
	*(con: bad programming)
	

	lea update_Tank_Return, a0  
  	jmp update_Tank	
update_Tank_Return:


	lea update_Turret_Return, a0  
  	jmp update_Turret	
update_Turret_Return:


	lea update_Enemies_Return, a0  
  	jmp update_Enemies	
update_Enemies_Return:


	lea update_Bullet_Return, a0  
  	jmp update_Bullet	
update_Bullet_Return:
	

	lea update_Power_Return, a0  
  	jmp update_PowerMeter	
update_Power_Return:

	lea collisions_Return, a0  
  	jmp checkForCollisions	
collisions_Return:
	*jsr	checkForCollisions



	movem.l (sp)+,ALL_REG
	rts

**************************************************************************
*Update Tank
**************************************************************************
update_Tank:


*Tank Position Previous = Current
	move.l TANK_X_CURRENT, TANK_X_PREVIOUS
	move.l TANK_Y_CURRENT, TANK_Y_PREVIOUS

*Removing the turret from the screen
	jsr blast_Turret

	move.l 	TANK_VELOCITY_X, d0
	move.l	TANK_X_CURRENT, d1

	move.l	ELASPED_TIME, d7	
	muls	d7, d0


	add.l	d0, d1
	move.l	d1, d2

	*lsr.l	#8, d2
	*lsr.l	#4, d2
	lsl.l	#4, d2
	swap	d2

	move.w	d2, d3

	cmp.w	#0, d2
	bgt	inBoundsLeft
	moveq.l	#0, d0


inBoundsLeft:
	move.l	#320, d4
	move.l	#TANK_WIDTH ,d5
	sub.l	d5, d4

	cmp	d4, d3
	blt	inBoundsRight
	moveq.l	#0, d0
inBoundsRight:
	
*Moving the Tank
	add.l	d0, TANK_X_CURRENT
		
	move.l	#0, TANK_VELOCITY_X

	jmp (a0)

***************************************************
*Update Turret
***************************************************
update_Turret:
		

	move.l	TURRET_ANGLE_CURRENT, TURRET_ANGLE_PREVIOUS
	
	move.l	TURRET_X_CURRENT, TURRET_X_PREVIOUS
	move.l	TURRET_Y_CURRENT, TURRET_Y_PREVIOUS

	jmp (a0)

*********************************************
*Update Power Meter
*********************************************
update_PowerMeter:


	
	move.l	POWER_METER_VALUE, d1
	move.l	#POWER_METER_SPEED, d2

	move.l	ELASPED_TIME, d3
	mulu	d3, d2

	add.w	d2, d1
		
	move.l	d1, POWER_METER_VALUE
	
	cmp.w	#$1000, d1
	blt	dontResetPowerMeterValue
	
	move.l	#1, POWER_METER_VALUE

dontResetPowerMeterValue:

	jmp (a0)





***************************************************
*Update Bullet
***************************************************
update_Bullet:


	move.l	#$1E0000, d0
	
*Checking if the bullet is below the stage, if so we can skip it
	move.l	BULLET_Y, d1
	cmp.l	d1, d0 
	blt	skipUpdateBullet

*Removing the old bullet from the screen
	jsr	blast_Bullet





*Adding Gravity to the Bullet's Y velocity
	move.l	#GRAVITY, d0
	move.l	ELASPED_TIME, d1
	muls	d1, d0
	add.l	d0, BULLET_VELOCITY_Y


*Adding the velocity to the position
	move.l	BULLET_VELOCITY_X, d1
	move.l	BULLET_VELOCITY_Y, d2

	asr.l	#4, d1
	asr.l	#4, d2
	move.l	ELASPED_TIME, d7	
	muls	d7, d1
	muls	d7, d2
	asl.l	#4, d1
	asl.l	#4, d2

	add.l	d1, BULLET_X
	add.l	d2, BULLET_Y

	addi.l	#3000, BULLET_FRAME
	move.l	BULLET_FRAME, d0

	lsl.l	#4, d0
	swap	d0
		

	cmp.w	#4, d0
	blt	reset_Bullet_Frame

	move.l	#0, BULLET_FRAME

reset_Bullet_Frame:
	
	jmp	escape_UpdateBullet	

skipUpdateBullet:
	move.l	#0, BULLET_FIRED 


escape_UpdateBullet:

	jmp (a0)






******************************************************
* Update Enemy
******************************************************
update_Enemies:
	movem.l ALL_REG, -(sp)

	jsr	blast_Enemies	

	moveq.l	#0, d0

*d0 = enemy loop counter
enemy_update_loop_start:

*Checking if we can leave the loop
	move.b	d0, d1	
	cmp.b	#NUMBER_OF_ENEMIES, d1
	bge	done_updating_enemies

			
	move.b	d0, d1
	addi.b	#1, d1
	

	

	lea	Enemy_Types, a1
	lea	Enemy_Frames, a2

	lea	Enemy_X_Pos, a3
	lea	Enemy_Y_Pos, a4	
	


	

	
	move.b	(a1, d0), d3	*Type
	move.b	(a2, d0), d4	*Frame


	*move.l	#0, d0


*Enemy X and Y Position



*Setting Previous = Current
	move.l	d0, d6
	asl.l	#2, d6
	move.l	(a3, d6), d4	*X
	move.l	(a4, d6), d5	*Y

	*add.l	d6, a4
	*add.l	d6, a5
	
	*move.l	d4, (a4)
	*move.l	d5, (a5)
	


*Getting rid of decimal places for drawing
	*asr.l	#8, d4
	*asr.l	#4, d4

	*asr.l	#8, d5
	*asr.l	#4, d5

	*move.l	#0, d0
	
	lea	Enemy_Timer, a6
	move.l	ELASPED_TIME, d7	
	*muls	d7, d2
	
	*move.l	#5,d2
	*cmp	d7, d2
	*blt	timeIsSafe
	*move.l	#5, d7
*timeIsSafe:

	*move.l	d0, d6
	*mulu	#4, d6
	addi.l	#1, (a6, d6) *d7

	move.l	(a6, d6), d1
	cmp.l	#$40, d1
	bgt	mode5

	
	move.l	(a6, d6), d1
	cmp.l	#$30, d1
	bgt	mode4


	move.l	(a6, d6), d1
	cmp.l	#$20, d1
	bgt	mode3


	move.l	(a6, d6), d1
	cmp.l	#$10, d1
	bgt	mode2


*Movement Mode 1
	move.l	#1000, d2
	move.l	#1000, d3
	move.l	ELASPED_TIME, d7	
	muls	d7, d2
	muls	d7, d3

	add.l	d2, d4
	add.l	d3, d5

	move.l	d4, (a3, d6)
	move.l	d5, (a4, d6)


	jmp	doneUpdatingEnemy

mode2:

*Movement Mode 2
	move.l	#1000, d2
	move.l	#-1000, d3
	move.l	ELASPED_TIME, d7	
	muls	d7, d2
	muls	d7, d3

	add.l	d2, d4
	add.l	d3, d5

	move.l	d4, (a3, d6)
	move.l	d5, (a4, d6)


	jmp	doneUpdatingEnemy

mode3:

*Movement Mode 3
	move.l	#-1000, d2
	move.l	#1000, d3
	move.l	ELASPED_TIME, d7	
	muls	d7, d2
	muls	d7, d3

	add.l	d2, d4
	add.l	d3, d5

	move.l	d4, (a3, d6)
	move.l	d5, (a4, d6)


	jmp	doneUpdatingEnemy

mode4:

*Movement Mode 4
	move.l	#-1000, d2
	move.l	#-1000, d3
	move.l	ELASPED_TIME, d7	
	muls	d7, d2
	muls	d7, d3

	add.l	d2, d4
	add.l	d3, d5

	move.l	d4, (a3, d6)
	move.l	d5, (a4, d6)


	
	jmp	doneUpdatingEnemy

mode5:

*Movement Mode 5
	move.l	#0, (a6, d6) 
	jmp	doneUpdatingEnemy


doneUpdatingEnemy:
	

	addi.b	#1, d0
	jmp	enemy_update_loop_start

	
done_updating_enemies:
	movem.l (sp)+,ALL_REG

	jmp (a0)



*****************************************************
* Collisions
*****************************************************
checkForCollisions:
	*movem.l ALL_REG, -(sp)
	move.l	#0, d0
	cmp.l	BULLET_FIRED,	d0
	bge	escape_bullet_collision


*d0 will be our loop counter	
	moveq.l	#0, d0

			
	move.b	d0, d1

	

	lea	Enemy_Types, a1
	lea	Enemy_Frames, a2
	lea	Enemy_X_Pos, a3
	lea	Enemy_Y_Pos, a4	
	lea	GiantBat_Size, a5
	




enemy_collision_loop_start:
	move.b	d0, d1	
	cmp.b	#NUMBER_OF_ENEMIES, d1
	bge	escape_bullet_collision
	


	*move.b	(a1, d0), d2	*Type
	*move.b	(a2, d0), d3	*Frame

*Enemy X and Y Position
	asl.l	#2, d0
	move.l	(a3, d0), d4	*X
	move.l	(a4, d0), d5	*Y
	asr.l	#1, d0

*Getting rid of decimal places for collisions check
	*asr.l	#8, d4
	*asr.l	#4, d4
	*asr.l	#8, d5
	*asr.l	#4, d5
	lsl.l	#4, d4
	swap	d4
	lsl.l	#4, d5
	swap	d5



	
	move.l	d0, d1
	addi.b 	#1, d1
	asl.b	#1, d1	

	move.w	(a5, d0), d6
	move.w	(a5, d1), d7
	add.w	d4, d6
	add.w	d5, d7


	sub.w	#8, d4
	sub.w	#8, d5
	addi.w	#8, d6
	addi.w	#8, d7

	
	move.l	BULLET_X, d2
	move.l	BULLET_Y, d3

	lsl.l	#4, d2
	swap	d2

	lsl.l	#4, d3
	swap	d3



*****************
*Note:
*	d2 = bullet x
*	d3 = bullet y
*
*	d4 = enemy X1
*	d5 = enemy Y1 
*	d6 = enemy X2
*	d7 = enemy Y2
*
******************

	move.w	d2, d1
	
*X1
	cmp.w	d4, d1 	
	blt	noCollision
*X2
	move.w	d2, d1
	cmp.w	d6, d1 	
	bgt	noCollision


	move.w	d3, d1
*Y1
	cmp.w	d5, d1
	blt	noCollision

*Y2
	move.w	d3, d1
	cmp.w	d7, d1 	
	bgt	noCollision


	addi.l	#1, HIT_COUNTER
	move.l	#1, BULLET_FIRED

	move.l	#$1E0000, d1
	move.l	d1, BULLET_Y


*Reseting Enemies When Hit
	move.b	d0, d1
	cmp	#0, d1
	bne	not1
	moveq.l	#0, d1
	move.b	d0, d1

	asl.l	#8, d1
	asl.l	#4, d1
	move.l	#$00020000, (a3, d1)
	move.l	#$00046000, (a4, d1)
	asr.l	#8, d1
	asr.l	#4, d1

not1:
	move.b	d0, d1
	cmp	#1, d1
	bne	not2
	moveq.l	#0, d1
	move.b	d0, d1
	
	asl.l	#8, d1
	asl.l	#4, d1
	move.l	#$000A0000, (a3, d1)
	move.l	#$00046000, (a4, d1)
	asr.l	#8, d1
	asr.l	#4, d1

		
not2:

noCollision:
	addi.b	#1, d0
	jmp	enemy_collision_loop_start


	
escape_bullet_collision:
	jmp (a0)





*****************************************************
* Blast Bullet
*****************************************************
blast_Bullet:
	move.l	BULLET_X, d1
	move.l	BULLET_Y, d2
*Removing Decimal Places
	lsl.l	#4, d1
	swap	d1
	lsl.l	#4, d2
	swap	d2




	sub.w	#16, d1
	sub.w	#16, d2

	pea    BackgroundData			*Push the address of the bitmap onto the stack

	move.w d1, -(SP)			*Push parameter the width on to the stack
    	move.w d2, -(SP)			*Push parameter the height on to the stack

	move.w #16, -(SP)			*Push parameter the width on to the stack
    	move.w #16, -(SP)			*Push parameter the height on to the stack
		
	move.w	d1, -(SP)			*Push parameter draw screen position y
    	move.w  d2, -(SP)			*Push parameter draw screen position y

	jsr	drawBMP				*Jump to sub routine drawBMP

	add.l #16, SP           		*Remove the passed parameters from the stack, all at once. (4 + (2 * 4))
	
	rts

*****************************************************
* Blast Enemies
*****************************************************
blast_Enemies:
	*movem.l ALL_REG, -(sp)

*Blasting Enemies

 *d0 is our loop variable
	moveq.l	#0, d0


	lea	Enemy_Types, a0
	lea	Enemy_Frames, a1
	lea	Enemy_X_Pos, a2
	lea	Enemy_Y_Pos, a3	

	lea	GiantBat_Size,   a5
	


enemy_blast_loop_start:
			
	move.b	d0, d1
	cmp.b	#NUMBER_OF_ENEMIES, d1
	bge	done_blasting_enemies


	
	move.b	(a0, d0), d2	*Type
	move.b	(a1, d0), d3	*Frame

*Enemy X and Y Position
	asl.l	#2, d0
	move.l	(a2, d0), d4	*X
	move.l	(a3, d0), d5	*Y
	asr.l	#2, d0



*Getting rid of decimal places for drawing
	*asr.l	#8, d4
	*asr.l	#4, d4
	lsl.l	#4, d4
	swap	d4



	*asr.l	#8, d5
	*asr.l	#4, d5
	lsl.l	#4, d5
	swap	d5
	


	*move.l	d0, d1
	*add.l 	#1, d1

	
	

	


	

	pea    BackgroundData			*Push the address of the bitmap onto the stack

	move.w d4, -(SP)			*Push parameter the width on to the stack
    	move.w d5, -(SP)			*Push parameter the height on to the stack


	move.l	d0, d1
	addi.b 	#1, d1
	asl.b	#1, d1	

	move.w	#0, d1
	move.w	#2, d2
	move.w	(a5, d1), d4
	move.w	(a5, d2), d5

	*add.w	#8, d4
	*add.w	#8, d5


	move.w d4, -(SP)			*Push parameter the width on to the stack
    	move.w d5, -(SP)			*Push parameter the height on to the stack
		

	asl.b	#2, d0
	move.l	(a2, d0), d4	*X
	move.l	(a3, d0), d5	*Y
	asr.b	#2, d0

	*asr.l	#8, d4
	*asr.l	#4, d4
	lsl.l	#4, d4
	swap	d4



	*asr.l	#8, d5
	*asr.l	#4, d5
	lsl.l	#4, d5
	swap	d5



	*sub.w	#8, d4
	*sub.w	#8, d5

	move.w d4, -(SP)			*Push parameter the width on to the stack
    	move.w d5, -(SP)			*Push parameter the height on to the stack

	jsr	drawBMP				*Jump to sub routine drawBMP
	add.l #16, SP           		*Remove the passed parameters from the stack, all at once. (4 + (2 * 4))


	addi.b	#1, d0
	jmp	enemy_blast_loop_start

done_blasting_enemies:

	*movem.l (sp)+,ALL_REG
	rts


	

*****************************************************
* Blast Turret
*****************************************************
blast_Turret:
	
*X1
	move.l	TANK_X_PREVIOUS, d1
	*lsr.l	#8, d1
	*lsr.l	#4, d1
	
	lsl.l	#4, d1
	swap	d1

	move.w	#TANK_WIDTH, d2
	lsr.w	#1, d2
	add.w	d2, d1


*Y1	
	move.l	TANK_Y_PREVIOUS, d2
	*lsr.l	#8, d2
	*lsr.l	#4, d2
	lsl.l	#4, d2
	swap	d2

	sub.w	#TURRET_LENGTH,  d2
	sub.w	#2, d2




*Blasting Previous Turrent
	pea    BackgroundData			*Push the address of the bitmap onto the stack
	
	move.w d1, -(SP)			*Push parameter positionX on to the stack	
 	move.w d2, -(SP)			*Push parameter positionY on to the stack
	
	move.w #TURRET_LENGTH, -(SP)		*Push parameter the width on to the stack
	move.w #TURRET_LENGTH, d3
	addi.w  #2, d3
    	move.w d3, -(SP)		*Push parameter the height on to the stack


	move.w d1, -(SP)			*Push parameter draw screen position x
    	move.w d2, -(SP)			*Push parameter draw screen position y

	jsr	drawBMP				*Jump to sub routine drawBMP

	add.l #16, SP           		*Remove the passed parameters from the stack, all at once. (4 + (2 * 4))

	rts





*****************************************************
*Renders
*****************************************************
render:
	movem.l ALL_REG, -(sp)

		
	

	jsr	draw_Turret

	jsr	draw_Tank
	
	jsr	draw_PowerMeter	

	jsr	draw_Score

	jsr	draw_Bullet
		
	jsr	draw_Enemies

		
	move.b	#SWAP_BUFFERS,	d0
	trap	#15


	movem.l (sp)+,ALL_REG
	rts


************
*Draw Tank
************
draw_Tank:

	
*Blasting Old Tank
	pea    BackgroundData			*Push the address of the bitmap onto the stack
	
	move.l (TANK_X_PREVIOUS), d1
	*lsr.l	#8, d1
	*lsr.l	#4, d1
	lsl.l	#4, d1
	swap	d1

	move.w  d1, -(SP)			*Push parameter positionX on to the stack
    	
	move.l (TANK_Y_CURRENT), d2		*Push parameter positionY on to the stack
	*lsr.l	#8, d2
	*lsr.l	#4, d2

	lsl.l	#4, d2
	swap	d2

	move.w  d2, -(SP)

	move.w #TANK_WIDTH, -(SP)		*Push parameter the width on to the stack
    	move.w #TANK_HEIGHT, -(SP)		*Push parameter the height on to the stack


	move.w d1, -(SP)			*Push parameter draw screen position x
    	move.w d2, -(SP)			*Push parameter draw screen position y

	jsr	drawBMP				*Jump to sub routine drawBMP

	add.l #16, SP           		*Remove the passed parameters from the stack, all at once. (4 + (2 * 4))
	
	*jsr	blast_Turret





*Draw New Tank

	pea    TankData				*Push the address of the bitmap onto the stack

	
	move.w #8, -(SP)			*Push parameter positionX on to the stack
    	move.w #1, -(SP)			*Push parameter positionY on to the stack


	move.w #TANK_WIDTH, -(SP)		*Push parameter the width on to the stack
    	move.w #TANK_HEIGHT, -(SP)		*Push parameter the height on to the stack
	


	
	
	move.l TANK_X_CURRENT, d1		*Push parameter draw screen position x
	*asr.l	#8, d1
	*asr.l	#4, d1
	lsl.l	#4, d1
	swap	d1


	move.w	d1, -(SP)


        *move.w #TANK_Y, -(SP)			*Push parameter draw screen position y
	move.l  TANK_Y_CURRENT, d2		
	*asr.l	#8, d2
	*asr.l	#4, d2
	lsl.l	#4, d2
	swap	d2

	move.w  d2, -(SP)



	jsr	drawBMP				*Jump to sub routine drawBMP

	add.l #16, SP           		*Remove the passed parameters from the stack, all at once. (4 + (2 * 4))


	rts



*******************
* Drawing Turret
*******************
draw_Turret:
	movem.l ALL_REG, -(sp)
	
	add.l	#-8, SP				*Creating 4 local word variables
	

TURRET_X1	equ	0
TURRET_Y1	equ	2
TURRET_X2	equ	4
TURRET_Y2	equ	6



*GETTING TURRET VALUES

	
*X1
	move.l	TANK_X_CURRENT, d1
	*lsr.l	#8, d1
	*lsr.l	#4, d1

	lsl.l	#4, d1
	swap	d1

	move.l	#TANK_WIDTH, d2
	lsr.l	#1, d2
	add.w	d2, d1
	move.w	d1, TURRET_X1(SP)

*Y1	
	move.l	TANK_Y_CURRENT, d2
	*lsr.l	#8, d2
	*lsr.l	#4, d2

	lsl.l	#4, d2
	swap	d2


	add.w	d4, d2
	move.w	d2, TURRET_Y1(SP)

	lea	SIN_TABLE, a3	
	lea	COS_TABLE, a4	

	move.w	TURRET_ANGLE_CURRENT, d7
	asl.l	#1, d7
*X2
	move.l	d1, d3
	move.l	#0, d5
	move.w	(a3, d7), d5

	*lsr.l	#8, d5
	*lsr.l	#4, d5
	
	mulu	#TURRET_LENGTH, d5
	*lsr.l	#8, d5
	*lsr.l	#4, d5
	lsl.l	#4, d5
	swap	d5


	add.l	d5, d3
	move.w	d3, TURRET_X2(SP)


*Y2
	move.l	d2, d4
	move.l	#0, d6

	move.w	(a4, d7), d6
	
	*lsr.l	#8, d6
	*lsr.l	#4, d6

	mulu	#TURRET_LENGTH, d6
	*lsr.l	#8, d6
	*lsr.l	#4, d6
	lsl.l	#4, d6
	swap	d6



	sub.w	d6, d4

	move.w	d4, TURRET_Y2(SP)





*Set Turret Color
	move.l	#TURRET_COLOR,d1
	move.b	#SET_PEN_COLOR,d0
	TRAP	#15

*Drawing Curret Turret
	move.w	TURRET_X1(SP), d1
	move.w	TURRET_Y1(SP), d2
	move.w	TURRET_X2(SP), d3
	move.w	TURRET_Y2(SP), d4


	asl.w	#1, d1
	asl.w	#1, d2
	asl.w	#1, d3
	asl.w	#1, d4


	move.b	#DRAW_LINE,d0
	TRAP #15

	add.l	#8, SP				*Removing the 4 local word variables

	movem.l (sp)+,ALL_REG
	rts


*********************
*Draw Power Meter
*********************
draw_PowerMeter:
	movem.l ALL_REG, -(sp)


*Set Power Meter Shell Color
	move.l	#POWER_METER_SHELL_COLOR, d1
	move.b	#SET_PEN_COLOR,d0
	TRAP	#15	

	move.l	#POWER_SHELL_FCOLOR, d1
	move.b	#SET_FILL_COLOR, d0
	TRAP	#15	


	move.w	#POWER_METER_X, d1
	move.w	#POWER_METER_Y, d2
	move.w	d1, d3
	move.w	d2, d4

	addi.w	#POWER_METER_WIDTH, d3
	addi.w	#POWER_METER_HEIGHT, d4
	
	move.b	#DRAW_RECTANGLE,d0
	TRAP #15




	move.l	#POWER_METER_SHELL_COLOR, d1
	move.b	#SET_PEN_COLOR, d0
	TRAP	#15	
	
	move.l	#POWER_METER_FILL_COLOR, d1
	move.b	#SET_FILL_COLOR, d0
	TRAP	#15	

	

	move.w	#POWER_METER_X, d1
	move.w	d1, d3

	move.l	POWER_METER_VALUE, d5
	move.l	#POWER_METER_WIDTH, d6
	asl.l	#8, d6
	asl.l	#4, d6
	*lsr.l	#4, d6
	*swap	d1
	

	mulu	#POWER_METER_WIDTH, d5
	*asr.l	#8, d5
	*asr.l	#4, d5
	lsl.l	#4, d5
	swap	d5
	
	add.w	d5, d3
	
	move.b	#DRAW_RECTANGLE,d0
	TRAP #15



	movem.l (sp)+,ALL_REG
	rts

    
*********************
*Draw Bullet
*********************
draw_Bullet:
	*movem.l ALL_REG, -(sp)

	move.l	#0, d0
	cmp.l	BULLET_FIRED,	d0
	bge	dontDrawBullet


*Draw New Bullet
	


	move.l	BULLET_X, d5
	move.l	BULLET_Y, d6


*Getting Rid of Decimal Places
	*asr.l	#8, d5
	*asr.l	#4, d5
	lsl.l	#4, d5
	swap	d5


	*asr.l	#8, d6
	*asr.l	#4, d6
	lsl.l	#4, d6
	swap	d6



	move.w	d1, d3
	move.w	d2, d4


	sub.w	#15, d5
	sub.w	#15, d6



	pea    BulletData			*Push the address of the bitmap onto the stack

	


	moveq.l	#0, d1
	moveq.l	#0, d2

	move.l	BULLET_FRAME, d0
	*asr.l	#8, d0
	*asr.l	#4, d0
	lsl.l	#4, d0
	swap	d0

	

	asl.l	#1, d0
	lea	AxeSprites,   a0
	
	
	move.b	(a0, d0), d1
	addi.b	#1, d0
	move.b	(a0, d0), d2


	move.w d1, -(SP)			*Push parameter the width on to the stack
    	move.w d2, -(SP)			*Push parameter the height on to the stack


	move.w #15, -(SP)			*Push parameter the width on to the stack
    	move.w #15, -(SP)			*Push parameter the height on to the stack
		

	move.w	d5, -(SP)			*Push parameter draw screen position y
    	move.w  d6, -(SP)			*Push parameter draw screen position y


	jsr	drawBMP				*Jump to sub routine drawBMP

	add.l #16, SP           		*Remove the passed parameters from the stack, all at once. (4 + (2 * 4))

dontDrawBullet:
	rts

************************************
* Draw Enemies - Targets
************************************	
draw_Enemies
	movem.l ALL_REG, -(sp)
	

	moveq.l	#0, d0

*d0 = enemy loop counter



	lea	Enemy_Types, a0
	lea	Enemy_Frames, a1
	lea	Enemy_X_Pos, a2
	lea	Enemy_Y_Pos, a3
	lea	GiantBat_Position,   a4
	lea	GiantBat_Size,   a5	



enemy_draw_loop_start:







*Draw Enemies				



*d0 = enemy number
	
	move.b	d0, d1	
	cmp.b	#NUMBER_OF_ENEMIES, d1
	bge	done_drawing_enemies

			
	*move.l	d0, d1
	*add.l	#1, d1

	
	*asl.l	#1, d1
	


	
	asl.b	#1, d0
	
	moveq.l	#0, d2 
	
	move.b	(a0, d0), d2	*Type
	move.l	d2, d3
	asl.b	#1, d3


	*move.b	(a1, d0), d3	*Frame
	
	asl.b	#1, d0

*Enemy X and Y Position
	move.l	(a2, d0), d4	*X
	move.l	(a3, d0), d5	*Y

	asr.b	#1, d0




*Getting rid of decimal places for drawing
	*asr.l	#8, d4
	*asr.l	#4, d4
	*asr.l	#8, d5
	*asr.l	#4, d5
	lsl.l	#4, d4
	swap	d4
	lsl.l	#4, d5
	swap	d5




	move.w	#0, d1
	move.w	(a4, d1), d6
	move.w	#2, d1
	move.w	(a4, d1), d7

	

	pea    EnemyData			*Push the address of the bitmap onto the stack

	move.w d6, -(SP)			*Push parameter the width on to the stack
    	move.w d7, -(SP)			*Push parameter the height on to the stack

	move.w	#0, d1
	move.w	(a5, d1), d6
	move.w	#2, d1
	move.w	(a5, d1), d7


	move.w d6, -(SP)			*Push parameter the width on to the stack
    	move.w d7, -(SP)			*Push parameter the height on to the stack
		


	move.w d4, -(SP)			*Push parameter the width on to the stack
    	move.w d5, -(SP)			*Push parameter the height on to the stack

	jsr	drawBMP				*Jump to sub routine drawBMP
	add.l #16, SP           		*Remove the passed parameters from the stack, all at once. (4 + (2 * 4))

	
	
	add.b	#1, d0
	jmp	enemy_draw_loop_start



done_drawing_enemies:	


		
	movem.l (sp)+,ALL_REG
	rts


*********************
*Score
*********************
draw_Score:
	movem.l ALL_REG, -(sp)

	moveq   #0, d3
	moveq   #0, d4


	move.w	#SCORE_LOCATION_X, d4
	move.w	#SCORE_LOCATION_Y, d5


*Blasting Score
	pea    BackgroundData				*Push the address of the bitmap onto the stack

	move.w d4, -(SP)			*Push parameter the width on to the stack
    	move.w d5, -(SP)			*Push parameter the height on to the stack

	move.w #100, -(SP)				*Push parameter the width on to the stack
    	move.w #30, -(SP)				*Push parameter the height on to the stack

	move.w	d4, -(SP)
    	move.w  d5, -(SP)		*Push parameter draw screen position y

	jsr	drawBMP					*Jump to sub routine drawBMP

	add.l #16, SP           			*Remove the passed parameters from the stack, all at once. (4 + (2 * 4))


*Setting the text color
		
	moveq.l	#BLACK, d1
	move.b	#SET_PEN_COLOR, d0
	TRAP	#15
	
	move.l	#TEXT_COLOR, d1
	move.b	#SET_FILL_COLOR, d0
	TRAP	#15



	*move.w	#SCORE_LOCATION_X, d4
	*move.w	#SCORE_LOCATION_Y, d5


*Converting Hex to Decmial
	move.l	HIT_COUNTER, d0
	jsr	Convert_Hex_to_BCD

	andi.l	#$000f, d0
	lea	SevenSegmentTable, a4
	moveq   #0, d7

	move.b	(a4, d0), d7



*Ones	
	addi.w  #50, d4
	move.w d4, -(SP)			*Position x
    	move.w d5, -(SP)			*Position y
	move.w d7, -(SP)			*Bits

	jsr	drawDigit
	
	add.l   #6, SP           		*Remove the passed parameters from the stack, all at once. (4 + (2 * 4))
*Tens
	
	move.l	HIT_COUNTER, d0
	jsr	Convert_Hex_to_BCD

	and.l	#$00f0, d0
	asr.l	#4, d0
	moveq   #0, d7

	move.b	(a4, d0), d7


	sub.w  #20, d4
	move.w d4, -(SP)			*Position x
    	move.w d5, -(SP)			*Position y
	move.w d7, -(SP)			*Bits

	jsr	drawDigit
	
	add.l   #6, SP    






*HITS

	*Converting Hex to Decmial
	move.l	BULLET_COUNTER, d0
	jsr	Convert_Hex_to_BCD

	and.l	#$000f, d0
	lea	SevenSegmentTable, a4
	moveq   #0, d7

	move.b	(a4, d0), d7



*Ones	
	addi.w  #70, d4
	move.w d4, -(SP)			*Position x
    	move.w d5, -(SP)			*Position y
	move.w d7, -(SP)			*Bits

	jsr	drawDigit
	
	add.l   #6, SP           		*Remove the passed parameters from the stack, all at once. (4 + (2 * 4))
*Tens
	
	move.l	BULLET_COUNTER, d0
	jsr	Convert_Hex_to_BCD

	andi.l	#$00f0, d0
	asr.l	#4, d0
	moveq   #0, d7

	move.b	(a4, d0), d7


	sub.w  #20, d4
	move.w d4, -(SP)			*Position x
    	move.w d5, -(SP)			*Position y
	move.w d7, -(SP)			*Bits

	jsr	drawDigit
	
	add.l   #6, SP    



*Drawing Slash

	move.l	#TEXT_COLOR, d1
	move.b	#SET_PEN_COLOR, d0
	TRAP	#15
	
*X
	move.w	d4, d1


	addi.w 	#61, d1

	move.w	d1, d3
	sub.w	#8, d3

*Y	
	move.w	#SCORE_LOCATION_Y, d7
	move.w	d7, d2
	move.w	d2, d4
	addi.w	#34, d4

	move.b	#DRAW_LINE,d0
	TRAP #15

	movem.l (sp)+,ALL_REG
	rts


*****************************************
* Convert Hex B to Dec
*****************************************
Convert_Hex_to_BCD:
	movem.l	d1,-(SP)		save registers

	moveq.l	#0,d1
	and.l	#$00FF,d0
	divu	#10,d0
	move.l	d0,d1
	asl.w	#4,d0			shift tens to proper location
	swap	d1				put units in proper location
	or.b	d1,d0			combine units with tens
	
	movem.l	(SP)+,d1		restore registers
	rts


**********************************************
* Draw Digit
**********************************************
DIGIT_X		EQU	68
DIGIT_Y		EQU	66
BITS		EQU	64

*REGISTERS = 4

*RA = 0
	


*	68 = Registers
*	
*	6 = Return Address from function
*

drawDigit:
	movem.l ALL_REG, -(sp)


	
*Storing Passed Values
	move.w	DIGIT_X(SP), d5
	move.w	DIGIT_Y(SP), d6
	move.w	BITS(SP), d7

*A
	move.w	d7, d2

	and.b	#%00000001,d2
	beq	skipA	

	move.w	#SCORE_LOCATION_X, d1
	addi.w	#2, d1
	add.w	d5, d1
	
	move.w	d1, d3
	addi.w	#10, d3


	move.w	#SCORE_LOCATION_Y, d2
	addi.w	#1, d2
	add.w	d6, d2

	move.w	d2, d4
	addi.b	#2, d4
	
	move.b	#DRAW_RECTANGLE,d0
	TRAP #15
skipA:

	
*B
	move.w	d7, d2
	and.b	#%00000010,d2
	beq	skipB	

	move.w	#SCORE_LOCATION_X, d1
	addi.w	#12, d1
	add.w	d5, d1
	
	move.w	d1, d3
	addi.w	#2, d3

	
	move.w	#SCORE_LOCATION_Y, d2
	addi.w	#3, d2
	add.w	d6, d2

	move.w	d2, d4
	add.b	#10, d4
	
	move.b	#DRAW_RECTANGLE,d0
	TRAP #15
skipB:

*C
	move.w	d7, d2
	and.b	#%00000100,d2
	beq	skipC	

	move.w	#SCORE_LOCATION_X, d1
	addi.w	#12, d1
	add.w	d5, d1
	
	move.w	d1, d3
	addi.w	#2, d3

	move.w	#SCORE_LOCATION_Y, d2
	addi.w	#14, d2
	add.w	d6, d2

	move.w	d2, d4
	addi.b	#10, d4
	
	move.b	#DRAW_RECTANGLE,d0
	TRAP #15
skipC:

*D
	move.w	d7, d2
	and.b	#%00001000,d2
	beq	skipD	

	move.w	#SCORE_LOCATION_X, d1
	addi.w	#2, d1
	add.w	d5, d1
	
	move.w	d1, d3
	addi.w	#10, d3

	move.w	#SCORE_LOCATION_Y, d2
	addi.w	#24, d2
	add.w	d6, d2

	move.w	d2, d4
	addi.b	#2, d4
	
	move.b	#DRAW_RECTANGLE,d0
	TRAP #15
skipD:

*E
	move.w	d7, d2
	and.b	#%00010000,d2
	beq	skipE	

	move.w	#SCORE_LOCATION_X, d1
	addi.w	#0, d1
	add.w	d5, d1
	
	move.w	d1, d3
	addi.w	#2, d3

	move.w	#SCORE_LOCATION_Y, d2
	addi.w	#14, d2
	add.w	d6, d2

	move.w	d2, d4
	addi.b	#10, d4
	
	move.b	#DRAW_RECTANGLE,d0
	TRAP #15
skipE:

*F
	move.w	d7, d2
	and.b 	#%00100000,d2
	beq	skipF	

	move.w	#SCORE_LOCATION_X, d1
	addi.w	#0, d1
	add.w	d5, d1
	
	move.w	d1, d3
	add.w	#2, d3

	move.w	#SCORE_LOCATION_Y, d2
	addi.w	#3, d2
	add.w	d6, d2

	move.w	d2, d4
	addi.b	#10, d4
	
	move.b	#DRAW_RECTANGLE,d0
	TRAP #15
skipF:

*G
	move.w	d7, d2
	and #%01000000,d2
	beq	skipG	

	move.w	#SCORE_LOCATION_X, d1
	addi.w	#2, d1
	add.w	d5, d1
	
	move.w	d1, d3
	addi.w	#10, d3

	move.w	#SCORE_LOCATION_Y, d2
	addi.w	#13, d2
	add.w	d6, d2

	move.w	d2, d4
	addi.b	#2, d4
	
	move.b	#DRAW_RECTANGLE,d0
	TRAP #15
skipG:


	


	
	movem.l (sp)+,ALL_REG		restore registers
	rts













********************************************************
*function - drawBMP
*
*Notes:
*	a0 = bitmapData
*
*	*d3 = The remainder of the width of the image divided by 4
*	d4 = current X pen location
*	d5 = current Y pen location
*	d6 = total image width
*	d7 = total image height
*
*
* Therefore we will start at (xPos, yPos) and make sure we draw at (xPos, (height - yPos)) so that we don't draw upside down
* We are using xPos yPos at the center of the image
********************************************************
drawBMP:
	movem.l ALL_REG, -(sp)

	add.l	#-10, SP				*Creating 3 local word variables

*******************************************************
*	Stack Pointer Variables
*
*
*	82 = BitmapData || a0
*	80 = xPos
*	78 = yPos
*	76 = width
*	74 = height
*
*	72 = (Screen Draw Position X)
*	70 = (Screen Draw Position Y)
*
*	68 = Registers
*	
*	6 = Return Address from function
*
*
*	4 = y0 (top y)
*	2 = x1 (right x)
*	0 = y1 (bottom y)
*
*******************************************************

BITMAP_DATA		EQU	86
X_POS			EQU	84
Y_POS			EQU	82
WIDTH			EQU	80
HEIGHT			EQU	78

SCREEN_DRAW_POSX	EQU	76
SCREEN_DRAW_POSY	EQU	74

CURRENT_X		EQU	8
RIGHTX			EQU	6
BOTTOMY			EQU	4

OFFSET			EQU	2

PEN_CURRENT_X		EQU	0



*Getting the right most x value	
	move.w  X_POS(SP), RIGHTX(SP)
	move.w	WIDTH(SP), d0
	add.w	d0, RIGHTX(SP)

*Getting the lower most y value 
	move.w  Y_POS(SP), BOTTOMY(SP)
	move.w	HEIGHT(SP), d0
	add.w	d0, BOTTOMY(SP)






	move.l	BITMAP_DATA(SP), a0		*Move the bitmapData address into a0

	move.w	(a0), d0			*moving the first word of bitmapData into d0
	move.w  #$424D, d1			*moving "BM" into d1
****************************************************************************************************
*Checking if it's actually a bitmap file (check if the first two bytes are "BM") if not escaping
****************************************************************************************************	
	cmp.w	d0, d1
	bne	drawBMP_EscapeFunction	



	add.l 	#14, a0 			*Moving to the end of the bitmap file header	
	add.l	#4, a0				*Moving to the width of the bitmap

*******************************************
*Changing the width to big edian formal
*******************************************
	move.l	(a0), d0
	jsr	switchLongBigEdian
	move.l  d0, d1


	move.l	d1, d6				*Putting the width of the image in d6 and in d3.  d6 is used for drawing and d3 is used for byte padding
	move.l	d6, d3
	
	divu.w	#4, d3				*Dividing the width by 4 and moving the remainder to the first word of d3
	
	asr.l	#8, d3				*This is used for later reference.
	asr.l	#8, d3
	move.w	d3, OFFSET(SP)
				
	asl.l	#8, d1				*Moving the width to the top word of d1, this is so we can set the screen size
	asl.l	#8, d1		


	add.l	#4, a0				*Moving to the height of the bitmap
	move.l  (a0), d0	
	
********************************************	
*Changing the height to big edian format
********************************************
	jsr	switchLongBigEdian
	move.l  d0, d2

	move.w	d2, d7				*storing the height of the image into d7 	
	add.w  d2, d1				*moving the height of the image into the lower word of d1.  The width of the image is in the upper word.


	*move.b	#SET_OUTPUT_WINDOW_SIZE, d0	*Setting the output window size
	*trap	#15


*******************************************************************************
*Setting the pen's curent location is (0, height of drawn image section)
*******************************************************************************
	move.w	#0, PEN_CURRENT_X(SP)		*X	
	move.l	SCREEN_DRAW_POSX(SP), d5	*Y

*******************************************************************************
*Setting the (x,y) current image location is (0,height)
*	
*We will just reuse xPos and yPos  that are on the stack
*	since we don't have any need for them any more
*******************************************************************************
	move.l	#0, CURRENT_X(SP) *X



****************************************
*Moving to the actual color data
****************************************
	add.l	#32, a0	


***************************************************************************************************
*
*	NOTE:	If the lowest row we are drawing is not the total image height then we need to 
* 		loop through the image going up rows until we reach the correct height
*
***************************************************************************************************


***************************************************
*Checking if we reached the end of this row
***************************************************
nextPixel:	
	*cmp	22(SP), d6
	*beq	ResetEndWidth



***************************************************
*Checking if we can skip drawing this pixel/Row
***************************************************	
*Bottom Check
	move.w	d7, d0
	cmp	BOTTOMY(SP), d0	
	bgt	skipRows
	
*Top Check
	move.w	d7, d0
	cmp	Y_POS(SP), d0	
	ble	drawBMP_EscapeFunction

*Left Check
	move.w	CURRENT_X(SP), d0
	cmp	X_POS(SP), d0	
	blt	skipColumnLeft
	
*Right Check
	move.w	CURRENT_X(SP), d0
	cmp	RIGHTX(SP), d0	
	bgt	skipColumnRight



*************************************
*Setting the pen's color
*************************************

	moveq.l	 #0, d0				*clearing d0
	moveq.l	 #0, d1				*clearing d1
	moveq.l	 #0, d2				*clearing d2

	move.b	(a0)+, d1
	asl.l   #8, d1
	asl.l   #8, d1

	move.b	(a0)+, d2
	asl.l   #8, d2
	add.l   d2, d1

	move.b  (a0)+, d0
	add.l   d0, d1



	move.l	d1, d2
	cmp.l	#$00FF00FF, d2
	beq	incrementX
 

	move.l	#SET_PEN_COLOR, d0		
	trap	#15
	
	
	
*********************************
*Getting the pen's location
*********************************
	move.w	CURRENT_X(SP), d1		*moving the current x pixel location into d1
	move.w	d7, d2				*moving the current y pixel location into d2
	

	sub.w	X_POS(SP), d1
	ext.l	d1
	sub.w	Y_POS(SP), d2
	ext.l	d2






	add.w	SCREEN_DRAW_POSX(SP), d1	*moving the current x pixel location into d1
	add.w	SCREEN_DRAW_POSY(SP), d2	*moving the current y pixel location into d2
	
	*TEST!
	asl.l	#1, d1
	asl.l	#1, d2
	*TEST!

	move.w	d1, d3
	add.w	#1, d3
	move.w	d2, d4
	add.w	#1, d4


*********************************
*Drawing this pixel
*********************************
	move.b	#DRAW_RECTANGLE, d0		*draw pixel
	*move.b	#DRAW_PIXEL, d0			*draw pixel

	trap	#15




incrementX:
	add.w	#1, CURRENT_X(SP)		*Increasing the current x pixel location
	jmp	nextPixel




skipPixel:

	
skipRows:
	move.l	d6, d1
	mulu	#3, d1

	add.w	OFFSET(SP), d1			*Add in offset
	move.w	BOTTOMY(SP), d0
	move.w	d7, d2
	sub.w	d0, d2

	mulu	d2, d1				*Mult by amount of rows
	add.l	d1, a0

	sub.l   d2, d7				*Increasing the current x pixel location

	jmp	nextPixel


skipColumnLeft:	
	move.w	CURRENT_X(SP), d0
	move.w	X_POS(SP), d1
	
	sub.w	d0, d1
	

	add.w	d1, CURRENT_X(SP)
	mulu 	#3, d1	
	add.w	d1, a0
	
	jmp	nextPixel

skipColumnRight:	
	move.w	CURRENT_X(SP), d0
	move.w	d6, d1

	sub.w	d0, d1
	
	mulu 	#3, d1
	
	add.w	d1, a0
	move.l	#0, CURRENT_X(SP)	


*Adding Offset
	add.w	OFFSET(SP), a0
	subi.l	#1, d7
	
	jmp	nextPixel

*******************************************************************
*Decreasing the row that we are on in the image the row counter
*******************************************************************
	ble	drawBMP_EscapeFunction			*Done drawing the image




**********************************
*Exit the function
**********************************
drawBMP_EscapeFunction:
	add.l	#10, SP				*Removing the 4 local word variables from the stack
	movem.l (sp)+,ALL_REG
	rts













ProgramDone:
	MOVE.B	#9,D0
	TRAP	#15		Halt Simulator

*************************************
*Function - Switch a long big edian
*************************************	
switchLongBigEdian:
	ror.w #8, d0
	swap d0
	ror.w #8, d0
	rts


        STOP #$2000

********************************************************
*Game State
********************************************************
GAMESTATE	ds.w	1







***********************************************************
*Tank Variables
***********************************************************
TANK_X_CURRENT	ds.l	1
TANK_X_PREVIOUS	ds.l	1

TANK_Y_CURRENT	ds.l	1
TANK_Y_PREVIOUS	ds.l	1

TANK_VELOCITY_X ds.l	1
TANK_VELOCITY_Y ds.l	1

TANK_FRAME		ds.l	1




	


************************************************
*Power Meter
************************************************
POWER_METER_VALUE	ds.l	1


************************************************
*Turret Variables
************************************************
TURRET_ANGLE_CURRENT	ds.l	1
TURRET_ANGLE_PREVIOUS	ds.l	1

TURRET_X_CURRENT	ds.l	1
TURRET_Y_CURRENT	ds.l	1

TURRET_X_PREVIOUS	ds.l	1
TURRET_Y_PREVIOUS	ds.l	1


************************************************
*Bullet Vars
************************************************
BULLET_FIRED		ds.l	1

BULLET_X		ds.l	1
BULLET_Y		ds.l	1

BULLET_VELOCITY_X 	ds.l	1
BULLET_VELOCITY_Y 	ds.l	1

BULLET_FRAME		ds.l	1

*************************************************
*Score
*************************************************
BULLET_COUNTER		ds.l	1
HIT_COUNTER		ds.l	1




*************************************************
*Time
*************************************************
CURRENT_TIME		ds.l	1
PREVIOUS_TIME		ds.l	1

ELASPED_TIME		ds.l	1



*******************
*Sin Table
*******************
SIN_TABLE	
		dc.w	$0000, $0047, $008E, $00D6, $011D, $0164, $01AC, $01F3, $023A, $0280, $02C7, $030D, $0353, $0399, $03DE, $0424, $0469, $04AD, $04F1, $0535 
		dc.w	$0578, $05BB, $05FE, $0640, $0681, $06C3, $0703, $0743, $0782, $07C1, $07FF, $083D, $087A, $08B6, $08F2, $092D, $0967, $09A1, $09D9, $0A11
		dc.w	$0A48, $0A7F, $0AB4, $0AE9, $0B1D, $0B50, $0B82, $0BB3, $0BE3, $0C13, $0C41, $0C6F, $0C9B, $0CC7, $0CF1, $0D1B, $0D43, $0D6B, $0D91, $0DB6 
		dc.w	$0DDB, $0DFE, $0E20, $0E41, $0E61, $0E80, $0E9D, $0EBA, $0ED5, $0EEF, $0F08, $0F20, $0F37, $0F4D, $0F61, $0F74, $0F86, $0F97, $0FA6, $0FB4
		dc.w	$0FC1, $0FCD, $0FD8, $0FE1, $0FE9, $0FF0, $0FF6, $0FFA, $0FFD, $0FFF, $1000

*******************
*Cos Table
*******************
COS_TABLE
		dc.w	$1000, $0FFF, $0FFD, $0FFA, $0FF6, $0FF0, $0FE9, $0FE1, $0FD8, $0FCD, $0FC1, $0FB4, $0FA6, $0F97, $0F86, $0F74, $0F61, $0F4D, $0F37, $0F20
		dc.w	$0F08, $0EEF, $0ED5, $0EBA, $0E9D, $0E80, $0E61, $0E41, $0E20, $0DFE, $0DDB, $0DB6, $0D91, $0D6B, $0D43, $0D1B, $0CF1, $0CC7, $0C9B, $0C6F
		dc.w	$0C41, $0C13, $0BE3, $0BB3, $0B82, $0B50, $0B1D, $0AE9, $0AB4, $0A7F, $0A48, $0A11, $09D9, $09A1, $0967, $092D, $08F2, $08B6, $087A, $083D
		dc.w	$0800, $07C1, $0782, $0743, $0703, $06C3, $0681, $0640, $05FE, $05BB, $0578, $0535, $04F1, $04AD, $0469, $0424, $03DE, $0399, $0353, $030D
		dc.w	$02C7, $0280, $023A, $01F3, $01AC, $0164, $011D, $00D6, $008E, $0047, $0000


*********************************	
*Key Input Function Table
*********************************
KeyInputFunctionTable   
                *0-9

		dc.l    routineSpace,0,0,0,0,0,0,0
		dc.l    0,0,0,0,0,0,0,0
                dc.l    0,0,0,0,0,0,0,0,0,0
                dc.l    0,0,0,0,0,0,0
                dc.l    routineA,0,0,routineD,0,0,0
                dc.l    0,0,0,0,0,0,0
                dc.l    0,0,routineQ,0,routineS,0,0
                dc.l    0,routineW,0,0,0


************************************************
*Seven Segment Table
************************************************
SevenSegmentTable
	dc.b  $3F			digit 0
	dc.b  $06			digit 1
	dc.b  $5B			digit 2
	dc.b  $4F			digit 3
	dc.b  $66			digit 4
	dc.b  $6D			digit 5
	dc.b  $7D			digit 6
	dc.b  $07			digit 7
	dc.b  $7F			digit 8
	dc.b  $6F			digit 9



************************************************
* Simon Sprite Sheet Positions
************************************************
SimonSprites
	dc.b	$30, $54		Simon Sprite 1
	dc.b	$59, $53		Simon Sprite 2
	dc.b	$80, $54		Simon Sprite 3
	dc.b	$A9, $53		Simon Sprite 4



************************************************
* Sub Weapons Sprite Sheet Positions
************************************************
AxeSprites
	dc.b	$30, $54		Axe Sprite 1
	dc.b	$59, $53		Axe Sprite 2
	dc.b	$80, $54		Axe Sprite 3
	dc.b	$A9, $53		Axe Sprite 4

	

**********************************************************
* Enemy Sprite Sheet Positions
**********************************************************
GiantBat_Position
	dc.w	$000C, $0171		Giant Bat Sprite 1
	dc.w	$0080, $0170		Giant Bat Sprite 1

**********************************************************
* Enemy Sprite Sheet Widths
**********************************************************
GiantBat_Size
	dc.w	$0030, $0016		Giant Bat Sprite 1
	dc.w	$0020, $0017		Giant Bat Sprite 1



**************************************************
* Enemy Tables
**************************************************
*What Enemy, Xpos, yPos

Enemy_Types
	dc.w	$0000
	dc.w	$0000

	
Enemy_Frames
	dc.w	$0000
	dc.w	$0000


Enemy_X_Pos
	dc.l	$00020000
	dc.l	$000A0000

Enemy_Y_Pos	
	dc.l	$00046000 	
	dc.l	$00046000


Enemy_Timer
	dc.l	$000000000
	dc.l	$000003000


********************************************************
*Bitmaps
********************************************************
TankData 	INCBIN "Castle.bmp"
BulletData 	INCBIN "Subweapons.bmp"

EnemyData	INCBIN "Enemy.bmp"

BackgroundData  INCBIN "stage1_background4.bmp"




*Background Data 2 is simply used for testing purposes
BackgroundData2  INCBIN "backgroundData2.bmp"

**************************************************
* Sounds
**************************************************
Music_1   
	dc.b    'WickedChild.wav',0   sound file


	END	START






































































































*~Font name~Courier New~
*~Font size~10~
*~Tab type~1~
*~Tab size~8~
